# cRangeLUT v1.0.0
# Written by Becky York
#
#Class that performs a x1-x2 to y1-y2 transformations
#based on a sequence of transformations
#
#Usage:
#                      x y x y x  y   x y  
#RangeLUT = cRangeLUT("1=1|4=5|6=4.5|10=1")
#print(RangeLUT.Translate(2.5)) ## 3
#print(RangeLUT.Translate(5.0)) ## 4.75
#print(RangeLUT.Translate(8.0)) ## 2.75
#
#there is no sort function
#please add items in order of x1->x2->x3->xN
class cRangeLUT:
    def __init__(self,strLUT):
        self.ranges = []
        if not strLUT == "":
            self._RangeFromLUT(strLUT)

    def _addRange(self, x, y):
        self.ranges.append([float(x),float(y)])

    def _RangeFromLUT(self,strLUT):
        for lrange in strLUT.split("|"):
            lValues = lrange.split("=")
            self._addRange(lValues[0],lValues[1])

    def mapRange(x,frX,toX,frY,toY):
        return (x - frX) * (toY - frY) / (toX - frX) + frY

    def Translate(self,x):
        flx = float(x)
        LastItem = len(self.ranges)-1
        if len(self.ranges) == 0:
            return 0
        if flx < self.ranges[0][0]:
            return self.ranges[0][1]
        if flx > self.ranges[LastItem][0]:
            return self.ranges[LastItem][1]
        #work backwards through the list
        for i in range(LastItem-1,-1,-1):
            frX = self.ranges[i][0]
            frY = self.ranges[i][1]
            toX = self.ranges[i+1][0]
            toY = self.ranges[i+1][1]
            if flx >= frX:
                return cRangeLUT.mapRange(flx,frX,toX,frY,toY)
        return -1
