from math import ceil
from time import time
from TyreBuddy_lib.sim_info import info

class cBuddy_Fuel():
    def __init__(self):
        self.Current_Lap_FuelStart = 0
        self.Current_Lap_TickStart = 0
        self.Fuel_Laps = [0,0,0,0,0]
        self.Fuel_Time = [0,0,0,0,0]
        self._OnLap = False
        self._FuelPerLap = 0
        self._FuelPerMinute = 0
        self.FuelIndex = 0

    @property
    def IsInPits(self):
        return (info.graphics.isInPit==1) or (info.graphics.isInPitLane==1)

    @property
    def Ticks(self):
        return round(time()*1000,0)

    @property
    def Fuel(self):
        return info.physics.fuel

    @property
    def MaxFuel(self):
        return info.static.maxFuel

    @property
    def FuelPerLap(self):
        return self._FuelPerLap

    @property
    def FuelPerMinute(self):
        return self._FuelPerMinute

    @property
    def LapsRemaining(self):
        if self._FuelPerLap == 0:
            return -1
        return self.Fuel / self._FuelPerLap

    @property
    def MinutesRemaining(self):
        if self._FuelPerMinute == 0:
            return -1
        return self.Fuel / self._FuelPerMinute

    @property
    def RaceLapsRemaining(self):
        return info.graphics.numberOfLaps - info.graphics.completedLaps

    @property
    def FuelRequiredForCurrentLap(self):
        return self._FuelPerLap * (1-info.graphics.normalizedCarPosition)

    @property
    def RefuelLitres(self):
        if self._FuelPerLap == 0:
            return -1

        numberOfLaps = self.RaceLapsRemaining
        sessionTimeLeft = round(info.graphics.sessionTimeLeft / 60000,2) #time left in minutes
        FuelRequired = -1
        if numberOfLaps > 0:
            FuelRequired = ((numberOfLaps-1) * self._FuelPerLap) + self._FuelPerLap
        elif sessionTimeLeft > 0:
            FuelRequired = (sessionTimeLeft * self._FuelPerMinute) + self._FuelPerLap

        #remove current fuel from required fuel
        if FuelRequired > 0:
            FuelRequired = FuelRequired - self.Fuel + self.FuelRequiredForCurrentLap

        return int(ceil(FuelRequired))

    @property
    def NextRefuelLitres(self):
        RefuelLitres = self.RefuelLitres
        if RefuelLitres == -1:
            return -1
        elif RefuelLitres > self.MaxFuel:
            return int(self.MaxFuel)
        else:
            return int(ceil(RefuelLitres))

    def InPits(self):
        self._OnLap = False

    def Demo(self,Fuel,Ticks):
        if self.Current_Lap_FuelStart == 0:
            self.Current_Lap_FuelStart = Fuel
            self.Current_Lap_TickStart = Ticks
        else:
            self.Fuel_Laps[self.FuelIndex] = self.Current_Lap_FuelStart - Fuel
            CurrentMinutes = (Ticks - self.Current_Lap_TickStart) / 60000 #ms to minutes
            self.Fuel_Time[self.FuelIndex] = self.Fuel_Laps[self.FuelIndex] / CurrentMinutes
            
            self.FuelIndex = ( self.FuelIndex + 1 ) % 5

            self.Calculate()

            self.Current_Lap_FuelStart = Fuel
            self.Current_Lap_TickStart = Ticks

    def LapStart(self):
        self.Current_Lap_FuelStart = self.Fuel
        self.Current_Lap_TickStart = self.Ticks
        self._OnLap = True

    def LapEnd(self,isTouristenfahrten):
        if self.IsInPits:
            return
        
        if not self._OnLap:
            self.LapStart()
            self._OnLap = True
            return

        self.Fuel_Laps[self.FuelIndex] = self.Current_Lap_FuelStart - self.Fuel
        CurrentMinutes = (self.Ticks - self.Current_Lap_TickStart) / 60000 #ms to minutes
        self.Fuel_Time[self.FuelIndex] = self.Fuel_Laps[self.FuelIndex] / CurrentMinutes
        
        self.FuelIndex = ( self.FuelIndex + 1 ) % 5
        
        self.Calculate()
        
        if not isTouristenfahrten:
            self.LapStart()

    def Calculate(self):
        ValidLaps = [x for x in self.Fuel_Laps if x > 0]
        ValidTimes = [x for x in self.Fuel_Time if x > 0]
        self._FuelPerLap = sum(ValidLaps) / len(ValidLaps)
        self._FuelPerMinute = sum(ValidTimes) / len(ValidTimes)
